@@ -10,6 +10,7 @@ exact problem is.
 """
 
 import sys
+import os
 from importlib.metadata import version, PackageNotFoundError
 
 import click
@@ -26,20 +27,20 @@ except PackageNotFoundError:
 
 def check_latest_version():
     """ check for later versions on pypi """
+    def echo(msg):
+        if not os.getenv('OUTDATED_IGNORE'):
+            click.secho(msg, bold=True, file=sys.stderr)
+
     try:
         is_outdated, latest_version = check_outdated(PACKAGE_NAME, __version__)
         if is_outdated:
-            click.secho(
-                f'Your local version ({__version__}) is out of date! Latest is {latest_version}!',
-                bold=True, file=sys.stderr,
-            )
+            echo(f'Your local version ({__version__}) is out of date! Latest is {latest_version}!')
     except ValueError:
         # this happens when your local version is ahead of the pypi version,
         # which happens only in development
         pass
     except KeyError:
-        # this happens when in dev mode, message could be removed
-        click.secho('Could not determine package version, continue nevertheless.', file=sys.stderr)
+        echo('Could not determine package version, continue nevertheless.')
         pass
 
 CLICK_CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])
@@ -187,14 +188,22 @@ def run(command, verbose, debug, dry_run, no_lock, update_source,
     help='Run the command step by step and stop when an error occurs (where applicable)',
     show_default=True
     )
+@click.option('--auto-approve/--no-auto-approve', is_flag=True, default=True,
+    help='Do not ask for confirmation before applying planned changes (where applicable)',
+    show_default=True
+    )
 @click.option(
     '--working-dir', '-w', default=None,
     help='Working directory, when omitted the current directory is used',
     )
+@click.option(
+    '--limit-parallelism', '-l', type=int, default=None,
+    help='Limit the parallelism to the given number, unlimitted if omitted',
+    )
 @click.argument('terragrunt-args', nargs=-1, type=click.UNPROCESSED)
 @click.version_option(version=__version__)
 def run_all(command, verbose, debug, dry_run, no_lock, update_source, exclude_external_dependencies,
-    step_by_step, working_dir, terragrunt_args):
+    step_by_step, auto_approve, working_dir, limit_parallelism, terragrunt_args):
     """ Executes a terragrunt command across multiple projects """
 
     check_latest_version()
@@ -208,7 +217,9 @@ def run_all(command, verbose, debug, dry_run, no_lock, update_source, exclude_ex
         update_source=update_source,
         exclude_external_dependencies=exclude_external_dependencies,
         step_by_step=step_by_step,
+        auto_approve=auto_approve,
         working_dir=working_dir,
+        limit_parallelism=limit_parallelism,
         terragrunt_args=terragrunt_args,
     )
 
@@ -321,8 +332,8 @@ def analyze(verbose, dry_run, exclude_external_dependencies, working_dir, terrag
     help='Target of config files',
     )
 @click.option(
-    '--project', '-p', default="",
-    help='Name of the project, if omitted all projects will be copied.',
+    '--module', '-m', default="",
+    help='Name of the module, if omitted all modules will be copied.',
     )
 @click.option('--clean-up', '-c', is_flag=True, default=False,
     help='Clean up files on target side that do not exist as source',
@@ -345,7 +356,7 @@ def analyze(verbose, dry_run, exclude_external_dependencies, working_dir, terrag
     help='Working directory, when omitted the current directory is used',
     )
 @click.version_option(version=__version__)
-def deploy(source_domain, source_stage, target_domain, target_stage, project, verbose, dry_run, clean_up, include_lock_file, working_dir):
+def deploy(source_domain, source_stage, target_domain, target_stage, module, verbose, dry_run, clean_up, include_lock_file, working_dir):
     """ Deploys the terragrunt config files from one stage to another (and possibly to a different domain) """
 
     check_latest_version()
@@ -356,9 +367,65 @@ def deploy(source_domain, source_stage, target_domain, target_stage, project, ve
         source_stage=source_stage,
         target_domain=target_domain,
         target_stage=target_stage,
-        project=project,
+        module=module,
         dry_run=dry_run,
         clean_up=clean_up,
         include_lock_file=include_lock_file,
         working_dir=working_dir,
     )
+
+@main.command(
+    name="git-deploy",
+    context_settings=dict(
+        ignore_unknown_options=True,
+    ),
+)
+@click.option(
+    '--git-repo', '-g', envvar='TERRAGRUNT_GIT_REPO',
+    help='URL of the git repository that contains the terragrunt files',
+    )
+@click.option(
+    '--source-directory', '-S',
+    help='Directory (in git repo) where terragrunt files are located',
+    )
+@click.option(
+    '--target-domain', '-T', default="",
+    help='Target domain where config files will be copied to, when omitted the project where you run this command is assumed.',
+    )
+@click.option(
+    '--target-stage', '-t',
+    type=click.Choice(STAGES, case_sensitive=True),
+    help='Target of config files',
+    )
+@click.option('--include-lock-file', '-i', is_flag=True, default=False,
+    help='Include the terraform lock file',
+    show_default=True
+    )
+@click.option('--verbose', '-v', is_flag=True, default=False,
+    help='Verbose printing',
+    show_default=True
+    )
+@click.option('--dry-run', '-D', is_flag=True, default=False,
+    help='dry-run mode, no real actions are executed',
+    show_default=True
+    )
+@click.option(
+    '--working-dir', '-w', default=None,
+    help='Working directory, when omitted the current directory is used',
+    )
+@click.version_option(version=__version__)
+def git_deploy(git_repo, source_directory, target_domain, target_stage, verbose, dry_run, include_lock_file, working_dir):
+    """ [experimental] Deploys the terragrunt config files from a git repository """
+
+    check_latest_version()
+
+    tgwrap = TgWrap(verbose=verbose)
+    tgwrap.git_deploy(
+        git_repo=git_repo,
+        source_directory=source_directory,
+        target_domain=target_domain,
+        target_stage=target_stage,
+        dry_run=dry_run,
+        include_lock_file=include_lock_file,
+        working_dir=working_dir,
+    )
