@@ -17,11 +17,11 @@ import sys
 import subprocess
 import shlex
 import shutil
-import string
 import re
 import tempfile
 import json
 import click
+import hcl
 import networkx as nx
 
 class Printer():
@@ -87,6 +87,54 @@ class TgWrap():
         """ Checks if a program is installed on the system """
         return shutil.which(program) is not None
 
+    def construct_command(self, command, debug, exclude_external_dependencies,
+        non_interactive=True, no_auto_approve=True, no_lock=True, update_source=False,
+        working_dir=None, limit_parallelism=None, terragrunt_args=()):
+        """ Constructs the command """
+        commands = {
+            'generic': '{base_command} {command} --terragrunt-non-interactive {no_auto_approve} {ignore_deps} {debug_level} {update_source} {working_dir}  {parallelism} {terragrunt_args}',
+            'info': '{base_command} terragrunt-info --terragrunt-non-interactive {ignore_deps} {debug_level} {update_source} {working_dir} {terragrunt_args}',
+            'plan': '{base_command} {command} --terragrunt-non-interactive  -out=planfile {ignore_deps} {debug_level} {lock_level} {update_source} {working_dir}  {parallelism} {terragrunt_args}',
+            'apply': '{base_command} {command} {non_interactive} {no_auto_approve} {parallelism} {ignore_deps} {debug_level} {update_source} {working_dir}  {parallelism} {terragrunt_args}',
+            'show': '{base_command} {command} --terragrunt-non-interactive {ignore_deps} {update_source} -json planfile', # no working dir!!!
+            'destroy': '{base_command} {command} --terragrunt-non-interactive --terragrunt-no-auto-approve {ignore_deps} {debug_level} {working_dir} {parallelism} {terragrunt_args}',
+        }
+
+        lock_stmt         = '-lock=false' if no_lock else ''
+        update_stmt       = '--terragrunt-source-update' if update_source else ''
+        ignore_deps_stmt  = '--terragrunt-ignore-external-dependencies' if exclude_external_dependencies else '--terragrunt-include-external-dependencies'
+        debug_stmt        = '--terragrunt-log-level debug --terragrunt-debug' if debug else ''
+        auto_approve_stmt = '--terragrunt-no-auto-approve' if no_auto_approve else ''
+        interactive_stmt  = '--terragrunt-non-interactive' if non_interactive else ''
+        working_dir_stmt  = f'--terragrunt-working-dir {working_dir}' if working_dir else ''
+        parallelism_stmt  = f'--terragrunt-parallelism {limit_parallelism}' if limit_parallelism else ''
+
+        base_command      = 'terragrunt run-all'
+
+        if command not in ['clean']:
+            full_command = commands.get(command, commands.get('generic')).format(
+                base_command=base_command,
+                command=command,
+                lock_level=lock_stmt,
+                update_source=update_stmt,
+                ignore_deps=ignore_deps_stmt,
+                debug_level=debug_stmt,
+                no_auto_approve=auto_approve_stmt,
+                non_interactive=interactive_stmt,
+                working_dir=working_dir_stmt,
+                parallelism=parallelism_stmt,
+                terragrunt_args=' '.join(terragrunt_args),
+            )
+        else:
+            full_command = commands.get(command, commands.get('generic'))
+
+        # remove double spaces
+        full_command = re.sub(' +', ' ', full_command)
+
+        self.printer.verbose(f'Full command to execute:\n$ {full_command}')
+
+        return full_command
+
     def prepare_groups(self, graph, exclude_external_dependencies):
         """ Prepare the list of groups that will be executed """
         groups = []
@@ -193,26 +241,25 @@ class TgWrap():
 
                             messages = ""
 
-                            f = tempfile.NamedTemporaryFile(mode='w+', prefix='tgwrap-', delete=True)
-                            self.printer.verbose(f"Opened temp file for error collection: {f.name}")
+                            with tempfile.NamedTemporaryFile(mode='w+', prefix='tgwrap-', delete=False) as f:
+                                self.printer.verbose(f"Opened temp file for error collection: {f.name}")
 
-                            rc = {'returncode': 0}
-                            rc = subprocess.run(
-                                shlex.split(command),
-                                text=True,
-                                cwd=directory,
-                                stdout=collect_output_file if collect_output_file else sys.stdout,
-                                stderr=f,
-                            )
-                            self.printer.verbose(rc)
+                                rc = {'returncode': 0}
+                                rc = subprocess.run(
+                                    shlex.split(command),
+                                    text=True,
+                                    cwd=directory,
+                                    stdout=collect_output_file if collect_output_file else sys.stdout,
+                                    stderr=f,
+                                )
+                                self.printer.verbose(rc)
 
-                            f.flush()
-
-                            messages = f.read()
+                            with open(f.name, 'r') as f:
+                                messages = f.read()
 
                             if rc.returncode != 0 or 'error' in messages.lower():
                                 raise Exception(
-                                    'An error situation detected while processing the terragrunt graph.'
+                                    f'An error situation detected while processing the terragrunt dependencies graph'
                                     )
                         except FileNotFoundError:
                             self.printer.warning(f'Directory {directory} not found, continue')
@@ -224,60 +271,14 @@ class TgWrap():
                             stop_processing = True
                             break
                         finally:
-                            f.close()
+                            os.remove(f.name)
 
             if stop_processing:
                 break
 
-    def construct_command(self, command, debug, exclude_external_dependencies,
-        non_interactive=True, no_auto_approve=True, no_lock=True, update_source=False,
-        working_dir=None, terragrunt_args=()):
-        """ Constructs the command """
-        commands = {
-            'generic': '{base_command} {command} --terragrunt-non-interactive {no_auto_approve} {ignore_deps} {debug_level} {update_source} {working_dir} {terragrunt_args}',
-            'info': '{base_command} terragrunt-info --terragrunt-non-interactive {ignore_deps} {debug_level} {update_source} {working_dir} {terragrunt_args}',
-            'plan': '{base_command} {command} --terragrunt-non-interactive  -out=planfile {ignore_deps} {debug_level} {lock_level} {update_source} {working_dir} {terragrunt_args}',
-            'apply': '{base_command} {command} {non_interactive} {no_auto_approve} --terragrunt-parallelism 1 {ignore_deps} {debug_level} {update_source} {working_dir} {terragrunt_args}',
-            'show': '{base_command} {command} --terragrunt-non-interactive {ignore_deps} {update_source} -json planfile', # no working dir!!!
-            'destroy': '{base_command} {command} --terragrunt-non-interactive --terragrunt-no-auto-approve {ignore_deps} {debug_level} {working_dir} {terragrunt_args}',
-        }
-
-        lock_stmt         = '-lock=false' if no_lock else ''
-        update_stmt       = '--terragrunt-source-update' if update_source else ''
-        ignore_deps_stmt  = '--terragrunt-ignore-external-dependencies' if exclude_external_dependencies else '--terragrunt-include-external-dependencies'
-        debug_stmt        = '--terragrunt-log-level debug --terragrunt-debug' if debug else ''
-        auto_approve_stmt = '--terragrunt-no-auto-approve' if no_auto_approve else ''
-        interactive_stmt  = '--terragrunt-non-interactive' if non_interactive else ''
-        working_dir_stmt  = f'--terragrunt-working-dir {working_dir}' if working_dir else ''
-
-        base_command      = 'terragrunt run-all'
-
-        if command not in ['clean']:
-            full_command = commands.get(command, commands.get('generic')).format(
-                base_command=base_command,
-                command=command,
-                lock_level=lock_stmt,
-                update_source=update_stmt,
-                ignore_deps=ignore_deps_stmt,
-                debug_level=debug_stmt,
-                no_auto_approve=auto_approve_stmt,
-                non_interactive=interactive_stmt,
-                working_dir=working_dir_stmt,
-                terragrunt_args=' '.join(terragrunt_args),
-            )
-        else:
-            full_command = commands.get(command, commands.get('generic'))
-
-        # remove double spaces
-        full_command = re.sub(' +', ' ', full_command)
-
-        self.printer.verbose(f'Full command to execute:\n$ {full_command}')
-
-        return full_command
-
     def run(self, command, debug, dry_run, no_lock, update_source,
         auto_approve, working_dir, terragrunt_args):
-        """ Executes a terragrunt command on a single project """
+        """ Executes a terragrunt command on a single module """
 
         self.printer.verbose(f"Attempting to execute 'run {command}'")
         if terragrunt_args:
@@ -288,7 +289,7 @@ class TgWrap():
             check_for_file = os.path.join(working_dir, check_for_file)
         if not os.path.isfile(check_for_file):
             self.printer.error(
-                f"{check_for_file} not found, this seems not to be a terragrunt project directory!"
+                f"{check_for_file} not found, this seems not to be a terragrunt module directory!"
                 )
             sys.exit(1)
 
@@ -313,8 +314,9 @@ class TgWrap():
             self.printer.verbose(rc)
 
     def run_all(self, command, debug, dry_run, no_lock, update_source,
-        exclude_external_dependencies, step_by_step, working_dir, terragrunt_args):
-        """ Executes a terragrunt command across multiple projects """
+        exclude_external_dependencies, step_by_step, auto_approve, working_dir,
+        limit_parallelism, terragrunt_args):
+        """ Executes a terragrunt command across multiple modules """
 
         self.printer.verbose(f"Attempting to execute 'run-all {command}'")
         if terragrunt_args:
@@ -324,23 +326,26 @@ class TgWrap():
             command=command,
             debug=debug,
             exclude_external_dependencies=True if step_by_step else exclude_external_dependencies,
-            non_interactive=step_by_step if command in ['apply', 'destroy'] else True,
+            # non_interactive=step_by_step if command in ['apply', 'destroy'] else True,
+            non_interactive=auto_approve,
             no_lock=no_lock,
             update_source=update_source,
-            no_auto_approve=False,
+            no_auto_approve=(not auto_approve),
             working_dir=working_dir,
             terragrunt_args=terragrunt_args,
+            limit_parallelism=limit_parallelism,
         )
 
         if step_by_step:
             self.printer.verbose(
-                f'This command will be executed in each individual project directory:\n$ {cmd}'
+                f'This command will be executed for each individual module:\n$ {cmd}'
                 )
             self.run_di_graph(
                 command=cmd,
                 exclude_external_dependencies=exclude_external_dependencies,
                 dry_run=dry_run,
                 working_dir=working_dir,
+                ask_for_confirmation=(not auto_approve),
             )
         else:
             if dry_run:
@@ -361,7 +366,7 @@ class TgWrap():
             check_for_file = os.path.join(working_dir, check_for_file)
         if not os.path.isfile(check_for_file):
             self.printer.error(
-                f"{check_for_file} not found, this seems not to be a terragrunt project directory!"
+                f"{check_for_file} not found, this seems not to be a terragrunt module directory!"
                 )
             sys.exit(1)
 
@@ -419,15 +424,16 @@ class TgWrap():
                     dry_run=dry_run,
                     collect_output_file=f,
                     working_dir=working_dir,
+                    ask_for_confirmation=False,
                 )
 
             with open(f.name, 'r') as f:
                 for line in f:
                     split_line = line.split(self.SEPARATOR)
-                    project = split_line[0]
+                    module = split_line[0]
                     plan_file = split_line[1]
 
-                    self.printer.header(f"Analyse project: {project}")
+                    self.printer.header(f"Analyse module: {module}")
 
                     try:
                         d = json.loads(plan_file)
@@ -484,7 +490,7 @@ class TgWrap():
 
                     except json.decoder.JSONDecodeError as e:
                         raise Exception(
-                            f"Planfile for {project} was no proper json, further analysis not possible."
+                            f"Planfile for {module} was no proper json, further analysis not possible."
                             ) from e
                     finally:
                         json_file.close()
@@ -492,42 +498,26 @@ class TgWrap():
             os.remove(f.name)
 
         if not ts_validation_successful:
-            raise Exception("Terrasafe validation failed on one or more projects")
+            raise Exception("Terrasafe validation failed on one or more modules")
 
-    def deploy(
-        self, dry_run, clean_up, include_lock_file, working_dir, 
-        source_stage, target_stage, source_domain, target_domain, project,
+    def run_sync(
+        self, source_path, target_path, source_stage, target_stage,
+        include_lock_file, dry_run, clean_up, git_source_path=None, source_domain=None,
         ):
-        """ Deploys the terragrunt config files from one stage to another (and possibly to a different domain) """
-    
-        if target_domain and not source_domain:
-            raise Exception("Providing a target domain while omitting the source domain is not supported!")
-        if source_domain and not target_domain:
-            raise Exception("Providing a source domain while omitting the target domain is not supported!")
+        """ Run a sync copying files from a source to a target path """
 
-        if target_domain and not target_stage:
-            self.printer.verbose(f"No target stage given, assume the same as source stage")
-            target_stage=source_stage
-
-        # do we have a working dir?
-        working_dir = working_dir if working_dir else os.getcwd()
-        # the domains will be ignored when omitted as input
-        source_path = os.path.join(working_dir, source_domain, source_stage, project, '')
-        target_path = os.path.join(working_dir, target_domain, target_stage, project, '')
-
-        # Check if source exists
         if not self.is_installed('rsync'):
             self.printer.error("'rsync' seems not installed. Cannot continue")
         elif not os.path.exists(source_path):
-            self.printer.error(f"Could not find {source_stage}/{project} in path {working_dir}. Cannot continue.")
+            self.printer.error(f"Cannot find {source_path}. Cannot continue.")
             if source_domain:
                 self.printer.error(
-                    "Please ensure you are in the directory that contains your landing zones, " + \
+                    "Please ensure you are in the directory that contains your projects, " + \
                     "or use --working-dir option"
                 )
             else:
                 self.printer.error(
-                    "Please ensure you are in the root of your landing zone, or use --working-dir option"
+                    "Please ensure you are in the root of your project, or use --working-dir option"
                 )
         else:
             self.printer.verbose(f"Copying config: {source_path} => {target_path}")
@@ -553,13 +543,131 @@ class TgWrap():
 
             cmd = re.sub(' +', ' ', cmd)
 
-            self.printer.normal("Will be deploying:")
-            self.printer.normal(f"from: {source_path}")
+            self.printer.header("Will be deploying:")
+            self.printer.normal(f"from: {git_source_path if git_source_path else source_path}")
             self.printer.normal(f"to:   {target_path}")
-            self.printer.verbose(f"\nUsing command:\n$ {cmd}")
+            self.printer.verbose(f"Using command:\n$ {cmd}")
             response = input("\nDo you want to continue? (y/n) ")
             if response.lower() != "y":
                 sys.exit(1)
 
             rc = subprocess.run(shlex.split(cmd))
             self.printer.verbose(rc)
+
+    def deploy(
+        self, source_stage, target_stage, source_domain, target_domain, module,
+        dry_run, clean_up, include_lock_file, working_dir, 
+        ):
+        """ Deploys the terragrunt config files from one stage to another (and possibly to a different domain) """
+    
+        if target_domain and not source_domain:
+            raise Exception("Providing a target domain while omitting the source domain is not supported!")
+        if source_domain and not target_domain:
+            raise Exception("Providing a source domain while omitting the target domain is not supported!")
+
+        if target_domain and not target_stage:
+            self.printer.verbose(f"No target stage given, assume the same as source stage")
+            target_stage=source_stage
+
+        # do we have a working dir?
+        working_dir = working_dir if working_dir else os.getcwd()
+        # the domains will be ignored when omitted as input
+        source_path = os.path.join(working_dir, source_domain, source_stage, module, '')
+        target_path = os.path.join(working_dir, target_domain, target_stage, module, '')
+
+        self.run_sync(
+            source_path=source_path,
+            target_path=target_path,
+            source_domain=source_domain,
+            source_stage=source_stage,
+            target_stage=target_stage,
+            include_lock_file=include_lock_file,
+            dry_run=dry_run,
+            clean_up=clean_up,
+        )
+
+    def git_deploy(
+        self, git_repo, source_directory, target_domain, target_stage,
+        dry_run, include_lock_file, working_dir, 
+        ):
+        """ Deploys the terragrunt config files from a git repository """
+        DEFAULT_VERSION='latest'
+
+        # first try to determine the desired version, this should be coded in
+        # an `version.hcl` file in the root of the target
+        version_file = os.path.join(os.getcwd(), target_domain, target_stage, "version.hcl")
+        version = DEFAULT_VERSION # default
+        if os.path.isfile(version_file):
+            self.printer.verbose(f'Try to get desired version info from {version_file}')
+            with open(version_file, 'r') as f:
+                version_info = hcl.load(f)
+
+            try:
+                version = version_info['locals'].get('platform_version')
+            except AttributeError as e:
+                click.warning(f'Could not determine platform version from file: {e}')
+        else:
+            self.printer.warning(f"Version file {version_file}'' not found, use '{DEFAULT_VERSION}' as version.")
+        
+        self.printer.verbose(f"Using version '{version}' as source")
+
+        try:
+            temp_dir = os.path.join(tempfile.mkdtemp(prefix='tgwrap-'), "tg-source")
+
+            # clone the repo
+            cmd = f"git clone {git_repo} {temp_dir}"
+            rc = subprocess.run(
+                shlex.split(cmd),
+                check=True,
+                )
+            self.printer.verbose(rc)
+
+            # now check out the specific version if we don't want latest
+            if version != DEFAULT_VERSION:
+                cmd = f"git checkout -b source {version}"
+                rc = subprocess.run(
+                    shlex.split(cmd),
+                    cwd=temp_dir,
+                    check=True,
+                    )
+                self.printer.verbose(rc)
+
+            # do we have a working dir?
+            working_dir = working_dir if working_dir else os.getcwd()
+
+            # run some sanity checks
+            reversed_path = list(reversed(source_directory.split('/')))
+            source_stage = reversed_path[0]
+            source_type = reversed_path[1]
+
+            if target_stage == 'global' and source_stage != target_stage:
+                self.printer.error(f'You are trying to copy source stage {source_stage} to a globa stage!')
+                sys.exit(1)
+
+            reversed_path = list(reversed(working_dir.split('/')))
+            target_type = reversed_path[0]
+            if target_type.upper() == 'DLZS' and not target_domain:
+                self.printer.warning(
+                    f'Detected that you are targetting a landing zone project but you provided no target domain!'
+                    )
+
+            # the source stage is either 'global' (if target == global) or 'dev'
+            source_stage = target_stage if target_stage == 'global' else 'dev'
+            # construct the paths
+            source_path = os.path.join(temp_dir, source_directory, '')
+            target_path = os.path.join(working_dir, target_domain, target_stage, '')
+
+            self.run_sync(
+                source_path=source_path,
+                git_source_path=f'{git_repo}//{source_directory}?ref={version}',
+                target_path=target_path,
+                source_stage=source_stage,
+                target_stage=target_stage,
+                include_lock_file=include_lock_file,
+                dry_run=dry_run,
+                clean_up=False,
+            )
+
+
+        finally:
+            shutil.rmtree(temp_dir)
\ No newline at end of file
